@@ -1,5 +1,5 @@
 use super::{
-    messages::{OrderInfo, OrderStatus, SubscriptionStatus, SystemStatus},
+    messages::{AddOrderRequest, OrderInfo, OrderStatus, SubscriptionStatus, SystemStatus},
     types::{BookData, SubscriptionType},
 };
 use futures::{
@@ -11,12 +11,12 @@ use std::{
     collections::{hash_map::Entry, HashMap, HashSet},
     str::FromStr,
     sync::{
-        atomic::{AtomicBool, Ordering},
+        atomic::{AtomicBool, AtomicU64, Ordering},
         Arc, Mutex,
     },
     time::{Duration, Instant},
 };
-use tokio::net::TcpStream;
+use tokio::{net::TcpStream, sync::oneshot};
 use tokio_tungstenite::{tungstenite::Message, MaybeTlsStream, WebSocketStream};
 use url::Url;
 
@@ -86,6 +86,14 @@ pub struct KrakenWsClient {
     // This is Some if we are subscribed, and contains the current sequence number
     // It is None if we are unsubscribed or there was an error
     open_orders_sequence_number: Option<u64>,
+    // Result senders for add_order calls
+    add_order_result_senders: HashMap<u64, oneshot::Sender<Result<String, String>>>,
+    // Result senders for cancel_order calls
+    cancel_order_result_senders: HashMap<u64, oneshot::Sender<Result<(), String>>>,
+    // Result senders for cancel_all_orders calls
+    cancel_all_orders_result_senders: HashMap<u64, oneshot::Sender<Result<u64, String>>>,
+    // Client req id ensures unique ids for different requests we make to kraken
+    client_req_id: AtomicU64,
 }
 
 impl KrakenWsClient {
@@ -103,9 +111,7 @@ impl KrakenWsClient {
     ///   up as you like.
     /// * Arc<WsApiResults>. This may be shared with synchronous code and polled for updates.
     ///   Note: KrakenWsAPI also conceals this detail.
-    pub async fn new(
-        config: KrakenWsConfig,
-    ) -> Result<(Self, SplitStream<WsClient>, Arc<WsAPIResults>), Error> {
+    pub async fn new(config: KrakenWsConfig) -> Result<(Self, SplitStream<WsClient>, Arc<WsAPIResults>), Error> {
         let url = if config.private.is_some() {
             Url::parse("wss://ws-auth.kraken.com").unwrap()
         } else {
@@ -129,13 +135,15 @@ impl KrakenWsClient {
             output: output.clone(),
             subscription_tracker: Default::default(),
             open_orders_sequence_number: None,
+            add_order_result_senders: Default::default(),
+            cancel_order_result_senders: Default::default(),
+            cancel_all_orders_result_senders: Default::default(),
+            client_req_id: Default::default(),
         };
 
         for pair in config.subscribe_book.iter() {
-            result
-                .subscription_tracker
-                .get_book(pair.to_string())
-                .last_request = Some((SubscriptionStatus::Subscribed, Instant::now()));
+            result.subscription_tracker.get_book(pair.to_string()).last_request =
+                Some((SubscriptionStatus::Subscribed, Instant::now()));
             result.subscribe_book(pair.to_string()).await?;
         }
 
@@ -165,7 +173,7 @@ impl KrakenWsClient {
                 self.handle_kraken_text(text);
             }
             Ok(Message::Binary(_)) => {
-                eprintln!("Warn: Unexpected binary message from Kraken");
+                log::warn!("Unexpected binary message from Kraken");
             }
             Ok(Message::Ping(_)) => {}
             Ok(Message::Pong(_)) => {}
@@ -184,38 +192,24 @@ impl KrakenWsClient {
     pub async fn check_subscriptions(&mut self) {
         // First look for active subscriptions with errors and try to unsubscribe
         for (asset_pair, sub) in self.subscription_tracker.book_subscriptions.iter_mut() {
-            if sub.status.is_subscribed()
-                && sub.needs_unsubscribe
-                && !sub.tried_to_change_recently()
-            {
+            if sub.status.is_subscribed() && sub.needs_unsubscribe && !sub.tried_to_change_recently() {
                 sub.last_request = Some((SubscriptionStatus::Unsubscribed, Instant::now()));
                 drop(sub);
-                if let Err(err) = Self::unsubscribe_book(
-                    &mut self.sink,
-                    self.config.book_depth,
-                    asset_pair.clone(),
-                )
-                .await
+                if let Err(err) =
+                    Self::unsubscribe_book(&mut self.sink, self.config.book_depth, asset_pair.clone()).await
                 {
-                    eprintln!(
-                        "Error: Could not unsubscribe from book {}: {}",
-                        asset_pair.clone(),
-                        err
-                    );
+                    log::error!("Could not unsubscribe from book {}: {}", asset_pair.clone(), err);
                 }
             }
         }
 
         {
             let mut sub = self.subscription_tracker.get_open_orders();
-            if sub.status.is_subscribed()
-                && sub.needs_unsubscribe
-                && !sub.tried_to_change_recently()
-            {
+            if sub.status.is_subscribed() && sub.needs_unsubscribe && !sub.tried_to_change_recently() {
                 sub.last_request = Some((SubscriptionStatus::Unsubscribed, Instant::now()));
                 drop(sub);
                 if let Err(err) = self.unsubscribe_open_orders().await {
-                    eprintln!("Error: Could not unsubscribe from open orders: {}", err);
+                    log::error!("Could not unsubscribe from open orders: {}", err);
                 }
             }
         }
@@ -225,14 +219,11 @@ impl KrakenWsClient {
         for asset_pair in self.config.subscribe_book.clone() {
             let mut sub = self.subscription_tracker.get_book(asset_pair.to_string());
             if !sub.status.is_subscribed() && !sub.tried_to_change_recently() {
-                eprintln!("Info: Resubscribing to book '{}'", asset_pair);
+                log::info!("Resubscribing to book '{}'", asset_pair);
                 sub.last_request = Some((SubscriptionStatus::Subscribed, Instant::now()));
                 drop(sub);
                 if let Err(err) = self.subscribe_book(asset_pair.to_string()).await {
-                    eprintln!(
-                        "Error: Could not subscribe to book '{}': {}",
-                        asset_pair, err
-                    );
+                    log::error!("Could not subscribe to book '{}': {}", asset_pair, err);
                 }
             }
         }
@@ -241,17 +232,145 @@ impl KrakenWsClient {
             if private_config.subscribe_open_orders {
                 let mut sub = self.subscription_tracker.get_open_orders();
                 if !sub.status.is_subscribed() && !sub.tried_to_change_recently() {
-                    eprintln!("Info: Resubscribing to openOrders");
+                    log::info!("Resubscribing to openOrders");
                     sub.last_request = Some((SubscriptionStatus::Subscribed, Instant::now()));
                     drop(sub);
                     if let Err(err) = self.subscribe_open_orders().await {
-                        eprintln!("Error: Could not subscribe to openOrders: {}", err);
+                        log::error!("Could not subscribe to openOrders: {}", err);
                     }
                 }
             }
         }
     }
 
+    /// Submit an order over the websocket
+    ///
+    /// The oneshot::Sender is sent Ok if the order is confirmed from Kraken,
+    /// and the TxID of the order is returned. The error message from kraken is
+    /// returned otherwise. The sender gets nothing if we fail to submit the order
+    /// at all.
+    pub async fn add_order(
+        &mut self,
+        mut order: AddOrderRequest,
+        result_sender: oneshot::Sender<Result<String, String>>,
+    ) -> Result<(), Error> {
+        let token = if let Some(private_config) = self.config.private.as_ref() {
+            private_config.token.clone()
+        } else {
+            log::error!("Tried to submit an order, but this is not an authenticated channel");
+            // Drop the result_sender and do not signal an error to the websocket
+            return Ok(());
+        };
+
+        let client_req_id = self.client_req_id.fetch_add(1, Ordering::SeqCst);
+        order.event = "addOrder".into();
+        order.reqid = Some(client_req_id);
+        order.token = token;
+
+        // This drops the result_sender if serialization or sending fails
+        match serde_json::to_string(&order) {
+            Err(err) => {
+                log::error!("Could not serialize order: {}", err);
+                return Ok(());
+            }
+            Ok(text) => {
+                // We have to store the result_sender before awaiting
+                self.add_order_result_senders.insert(client_req_id, result_sender);
+                self.sink.send(Message::Text(text)).await.map_err(|err| {
+                    self.add_order_result_senders.remove(&client_req_id);
+                    err
+                })?;
+            }
+        }
+        Ok(())
+    }
+
+    /// Submit a request to cancel an order over the websocket
+    ///
+    /// TxID may be a string used to identify an order, or a user-ref-id
+    ///
+    /// The oneshot::Sender is sent Ok if the cancel order is successful,
+    /// and the error message from kraken otherwise. The sender gets nothing
+    /// if we fail to submit the request at all.
+    pub async fn cancel_order(
+        &mut self,
+        txid: String,
+        result_sender: oneshot::Sender<Result<(), String>>,
+    ) -> Result<(), Error> {
+        let token = if let Some(private_config) = self.config.private.as_ref() {
+            private_config.token.clone()
+        } else {
+            log::error!("Tried to submit an order, but this is not an authenticated channel");
+            // Drop the result_sender and do not signal an error to the websocket
+            return Ok(());
+        };
+
+        let client_req_id = self.client_req_id.fetch_add(1, Ordering::SeqCst);
+
+        let payload = json! ({
+            "event": "cancelOrder",
+            "token": token,
+            "txid": [txid],
+            "reqid": client_req_id,
+        });
+
+        // We have to store the result_sender before awaiting
+        self.cancel_order_result_senders.insert(client_req_id, result_sender);
+
+        // This drops the result_sender if sending fails
+        self.sink
+            .send(Message::Text(payload.to_string()))
+            .await
+            .map_err(|err| {
+                self.cancel_order_result_senders.remove(&client_req_id);
+                err
+            })?;
+
+        Ok(())
+    }
+
+    /// Submit a request to cancel all orders over the websocket
+    ///
+    /// The oneshot::Sender is sent Ok if the cancel order is successful, with
+    /// the number of orders canceled. The sender gets the error message from
+    /// kraken otherwise. The sender gets nothing
+    /// if we fail to submit the request at all.
+    pub async fn cancel_all_orders(
+        &mut self,
+        result_sender: oneshot::Sender<Result<u64, String>>,
+    ) -> Result<(), Error> {
+        let token = if let Some(private_config) = self.config.private.as_ref() {
+            private_config.token.clone()
+        } else {
+            log::error!("Tried to submit an order, but this is not an authenticated channel");
+            // Drop the result_sender and do not signal an error to the websocket
+            return Ok(());
+        };
+
+        let client_req_id = self.client_req_id.fetch_add(1, Ordering::SeqCst);
+
+        let payload = json! ({
+            "event": "cancelAll",
+            "token": token,
+            "reqid": client_req_id,
+        });
+
+        // We have to store the result_sender before awaiting
+        self.cancel_all_orders_result_senders
+            .insert(client_req_id, result_sender);
+
+        // This drops the result_sender if sending fails
+        self.sink
+            .send(Message::Text(payload.to_string()))
+            .await
+            .map_err(|err| {
+                self.cancel_all_orders_result_senders.remove(&client_req_id);
+                err
+            })?;
+
+        Ok(())
+    }
+
     /// Close the socket gracefully
     pub async fn close(&mut self) -> Result<(), Error> {
         self.output.stream_closed.store(true, Ordering::SeqCst);
@@ -274,11 +393,7 @@ impl KrakenWsClient {
     /// Unsubscribe from a book stream
     ///
     /// Note: We made this not take self, to resolve a borrow checker issue
-    async fn unsubscribe_book(
-        sink: &mut SinkType,
-        book_depth: usize,
-        pair: String,
-    ) -> Result<(), Error> {
+    async fn unsubscribe_book(sink: &mut SinkType, book_depth: usize, pair: String) -> Result<(), Error> {
         let payload = json!({
             "event": "unsubscribe",
             "pair": [pair],
@@ -330,45 +445,48 @@ impl KrakenWsClient {
                 if let Some(event) = map.get("event") {
                     if event == "subscriptionStatus" {
                         if let Err(err) = self.handle_subscription_status(map) {
-                            eprintln!("Error: handling subscription status: {}\n{}", err, text)
+                            log::error!("handling subscription status: {}\n{}", err, text)
                         }
                     } else if event == "systemStatus" {
                         if let Err(err) = self.handle_system_status(map) {
-                            eprintln!("Error: handling system status: {}\n{}", err, text)
+                            log::error!("handling system status: {}\n{}", err, text)
+                        }
+                    } else if event == "addOrderStatus" {
+                        if let Err(err) = self.handle_add_order_status(map) {
+                            log::error!("handling add order status: {}\n{}", err, text)
+                        }
+                    } else if event == "cancelOrderStatus" {
+                        if let Err(err) = self.handle_cancel_order_status(map) {
+                            log::error!("handling cancel order status: {}\n{}", err, text)
+                        }
+                    } else if event == "cancelAllStatus" {
+                        if let Err(err) = self.handle_cancel_all_orders_status(map) {
+                            log::error!("handling cancel all order status: {}\n{}", err, text)
                         }
                     } else if event == "pong" || event == "heartbeat" {
                         // nothing to do
                     } else {
-                        eprintln!("Error: Unknown event from kraken: {}\n{}", event, text);
+                        log::error!("Unknown event from kraken: {}\n{}", event, text);
                     }
                 } else {
-                    eprintln!(
-                        "Error: Missing event string in payload from Kraken: {}",
-                        text
-                    );
+                    log::error!("Missing event string in payload from Kraken: {}", text);
                 }
             }
             Ok(Value::Array(array)) => {
                 if let Err(err) = self.handle_array(array) {
-                    eprintln!("Error: handling array payload: {}\n{}", err, text);
+                    log::error!("handling array payload: {}\n{}", err, text);
                 }
             }
             Ok(val) => {
-                eprintln!("Error: Unexpected json value from Kraken: {:?}", val);
+                log::error!("Unexpected json value from Kraken: {:?}", val);
             }
             Err(err) => {
-                eprintln!(
-                    "Error: Could not deserialize json from Kraken: {}\n{}",
-                    err, text
-                );
+                log::error!("Could not deserialize json from Kraken: {}\n{}", err, text);
             }
         }
     }
 
-    fn handle_subscription_status(
-        &mut self,
-        map: serde_json::Map<String, Value>,
-    ) -> Result<(), &'static str> {
+    fn handle_subscription_status(&mut self, map: serde_json::Map<String, Value>) -> Result<(), &'static str> {
         let status = SubscriptionStatus::from_str(
             map.get("status")
                 .ok_or("Missing status")?
@@ -382,7 +500,7 @@ impl KrakenWsClient {
                     .ok_or("missing errorMessage")?
                     .as_str()
                     .ok_or("errorMessage is not a string")?;
-                eprintln!("subscription error: {}", err_msg);
+                log::error!("subscription error: {}", err_msg);
                 return Err("subscription error");
             }
             SubscriptionStatus::Subscribed | SubscriptionStatus::Unsubscribed => {
@@ -414,14 +532,13 @@ impl KrakenWsClient {
                             .ok_or("pair was not a string")?
                             .clone();
 
-                        self.subscription_tracker
-                            .add_book_channel(channel_name.to_string());
+                        self.subscription_tracker.add_book_channel(channel_name.to_string());
                         let sub = self.subscription_tracker.get_book(pair.to_string());
                         if sub.status != status {
-                            eprintln!("{} @ {} book: {}", status, pair, channel_name);
+                            log::info!("{} @ {} book: {}", status, pair, channel_name);
                             *sub = SubscriptionState::new(status);
                         } else {
-                            eprintln!("Unexpected repeated {} message: {:?}", status, map);
+                            log::warn!("Unexpected repeated {} message: {:?}", status, map);
                         }
                     }
                     SubscriptionType::OpenOrders => {
@@ -429,14 +546,14 @@ impl KrakenWsClient {
                         if sub.status != status {
                             *sub = SubscriptionState::new(status);
                             if status.is_subscribed() {
-                                eprintln!("Subscribed to {}", channel_name);
+                                log::info!("Subscribed to {}", channel_name);
                                 self.open_orders_sequence_number = Some(0);
                             } else {
-                                eprintln!("Unsubscribed from {}", channel_name);
+                                log::info!("Unsubscribed from {}", channel_name);
                                 self.open_orders_sequence_number = None;
                             }
                         } else {
-                            eprintln!("Unexpected repeated {} message: {:?}", status, map);
+                            log::warn!("Unexpected repeated {} message: {:?}", status, map);
                         }
                     }
                 }
@@ -473,9 +590,7 @@ impl KrakenWsClient {
                     .ok_or("unexpected openOrders message")?;
                 if *our_seq_number + 1 != sequence_number {
                     // We need to try to resubscribe to open orders now
-                    self.subscription_tracker
-                        .get_open_orders()
-                        .needs_unsubscribe = true;
+                    self.subscription_tracker.get_open_orders().needs_unsubscribe = true;
                     return Err("openOrders sequence number mismatch");
                 }
                 *our_seq_number += 1;
@@ -488,10 +603,7 @@ impl KrakenWsClient {
                 .as_array()
                 .ok_or("updates were not an array")?;
             for update in updates {
-                for (order_id, val) in update
-                    .as_object()
-                    .ok_or("expected update to be an object")?
-                {
+                for (order_id, val) in update.as_object().ok_or("expected update to be an object")? {
                     match open_orders.entry(order_id.to_string()) {
                         Entry::Occupied(mut entry) => {
                             // This is likely a status update, lets see what to do
@@ -500,26 +612,23 @@ impl KrakenWsClient {
                                 .ok_or("order update was not an object")?
                                 .get("status")
                                 .ok_or("order update missing status")?;
-                            let status: OrderStatus = serde_json::from_value(status.clone())
-                                .map_err(|err| {
-                                    eprintln!("Could not parse order status: {}", err);
-                                    "OrderStatus deserialization error"
-                                })?;
+                            let status: OrderStatus = serde_json::from_value(status.clone()).map_err(|err| {
+                                log::error!("Could not parse order status: {}", err);
+                                "OrderStatus deserialization error"
+                            })?;
                             match status {
                                 OrderStatus::Pending | OrderStatus::Open => {
                                     entry.get_mut().status = status;
                                 }
-                                OrderStatus::Closed
-                                | OrderStatus::Expired
-                                | OrderStatus::Canceled => {
+                                OrderStatus::Closed | OrderStatus::Expired | OrderStatus::Canceled => {
                                     entry.remove();
                                 }
                             }
                         }
                         Entry::Vacant(entry) => {
                             // Parse the data as an OrderInfo object and add the new order id
-                            let order_info : OrderInfo = serde_json::from_value(val.clone()).map_err(|err| {
-                                eprintln!("Could not parse open order data as an OrderInfo object: {}", err);
+                            let order_info: OrderInfo = serde_json::from_value(val.clone()).map_err(|err| {
+                                log::error!("Could not parse open order data as an OrderInfo object: {}", err);
                                 "OrderInfo deserialization error"
                             })?;
                             entry.insert(order_info);
@@ -547,7 +656,7 @@ impl KrakenWsClient {
                 .output
                 .book
                 .get(pair)
-                .ok_or("missing asset pair in api results")?
+                .ok_or("unexpected asset pair update -- check asset pair name")?
                 .lock()
                 .expect("mutex poisoned");
 
@@ -584,20 +693,15 @@ impl KrakenWsClient {
                     }
                     // If we got a checksum, lets check it
                     if let Some(check_val) = obj.get("c") {
-                        let expected_checksum = u32::from_str(
-                            check_val
-                                .as_str()
-                                .ok_or("checksum value was not a string")?,
-                        )
-                        .map_err(|_| "checksum value could not parse as u32")?;
+                        let expected_checksum =
+                            u32::from_str(check_val.as_str().ok_or("checksum value was not a string")?)
+                                .map_err(|_| "checksum value could not parse as u32")?;
                         let checksum = book.checksum();
                         if checksum != expected_checksum {
-                            eprintln!("Error: checksum mismatch, book is out of sync.");
+                            log::error!("Error: checksum mismatch, book is out of sync.");
                             book.checksum_failed = true;
                             drop(book);
-                            self.subscription_tracker
-                                .get_book(pair.to_string())
-                                .needs_unsubscribe = true;
+                            self.subscription_tracker.get_book(pair.to_string()).needs_unsubscribe = true;
                             return Err("checksum mismatch");
                         }
                     }
@@ -611,10 +715,7 @@ impl KrakenWsClient {
         }
     }
 
-    fn handle_system_status(
-        &mut self,
-        map: serde_json::Map<String, Value>,
-    ) -> Result<(), &'static str> {
+    fn handle_system_status(&mut self, map: serde_json::Map<String, Value>) -> Result<(), &'static str> {
         let status = SystemStatus::from_str(
             map.get("status")
                 .ok_or("missing status field")?
@@ -624,6 +725,119 @@ impl KrakenWsClient {
         *self.output.system_status.lock().expect("mutex poisoned") = Some(status);
         Ok(())
     }
+
+    fn handle_add_order_status(&mut self, map: serde_json::Map<String, Value>) -> Result<(), &'static str> {
+        let req_id = map
+            .get("reqid")
+            .ok_or("missing req_id field")?
+            .as_u64()
+            .ok_or("reqid wasnt an integer")?;
+        let sender = self
+            .add_order_result_senders
+            .remove(&req_id)
+            .ok_or("unknown add_order reqid")?;
+        let status = map
+            .get("status")
+            .ok_or("missing status field")?
+            .as_str()
+            .ok_or("status wasnt a string")?;
+        if status == "ok" {
+            // tx_id is omitted when validate=true
+            let tx_id = map
+                .get("txid")
+                .map(|val| val.as_str().ok_or("txid wasnt a string"))
+                .transpose()?
+                .unwrap_or_default();
+            drop(sender.send(Ok(tx_id.to_string())));
+            Ok(())
+        } else if status == "error" {
+            let err_msg = map
+                .get("errorMessage")
+                .ok_or("missing errorMessage field")?
+                .as_str()
+                .ok_or("errorMessage wasnt a string")?;
+            log::error!("add_order: {}", err_msg);
+            drop(sender.send(Err(err_msg.to_string())));
+            Ok(())
+        } else {
+            log::error!("unexpected status: {}", status);
+            drop(sender.send(Err(format!("unexpected status: {}", status))));
+            Err("unexpected status")
+        }
+    }
+
+    fn handle_cancel_order_status(&mut self, map: serde_json::Map<String, Value>) -> Result<(), &'static str> {
+        let req_id = map
+            .get("reqid")
+            .ok_or("missing req_id field")?
+            .as_u64()
+            .ok_or("reqid wasnt an integer")?;
+        let sender = self
+            .cancel_order_result_senders
+            .remove(&req_id)
+            .ok_or("unknown cancel_order reqid")?;
+        let status = map
+            .get("status")
+            .ok_or("missing status field")?
+            .as_str()
+            .ok_or("status wasnt a string")?;
+        if status == "ok" {
+            drop(sender.send(Ok(())));
+            Ok(())
+        } else if status == "error" {
+            let err_msg = map
+                .get("errorMessage")
+                .ok_or("missing errorMessage field")?
+                .as_str()
+                .ok_or("errorMessage wasnt a string")?;
+            log::error!("cancel_order: {}", err_msg);
+            drop(sender.send(Err(err_msg.to_string())));
+            Ok(())
+        } else {
+            log::error!("unexpected status: {}", status);
+            drop(sender.send(Err(format!("unexpected status: {}", status))));
+            Err("unexpected status")
+        }
+    }
+
+    fn handle_cancel_all_orders_status(&mut self, map: serde_json::Map<String, Value>) -> Result<(), &'static str> {
+        let req_id = map
+            .get("reqid")
+            .ok_or("missing req_id field")?
+            .as_u64()
+            .ok_or("reqid wasnt an integer")?;
+        let sender = self
+            .cancel_all_orders_result_senders
+            .remove(&req_id)
+            .ok_or("unknown cancel_all_orders reqid")?;
+        let status = map
+            .get("status")
+            .ok_or("missing status field")?
+            .as_str()
+            .ok_or("status wasnt a string")?;
+        if status == "ok" {
+            let count = map
+                .get("count")
+                .ok_or("missing count field")?
+                .as_u64()
+                .ok_or("count wasnt an integer")?;
+            drop(sender.send(Ok(count)));
+            Ok(())
+        } else if status == "error" {
+            let err_msg = map
+                .get("errorMessage")
+                .ok_or("missing errorMessage field")?
+                .as_str()
+                .ok_or("errorMessage wasnt a string")?;
+            log::error!("cancel_all_orders: {}", err_msg);
+            drop(sender.send(Err(err_msg.to_string())));
+            Ok(())
+        } else {
+            log::error!("unexpected status: {}", status);
+            drop(sender.send(Err(format!("unexpected status: {}", status))));
+            Err("unexpected status")
+        }
+    }
 }
 
 impl Drop for KrakenWsClient {
@@ -687,9 +901,7 @@ impl SubscriptionState {
     /// rather than try to change it again.
     pub fn tried_to_change_recently(&self) -> bool {
         self.last_request
-            .map(|(stat, time)| {
-                stat != self.status && time + SUBSCRIPTION_CHANGE_BACKOFF > Instant::now()
-            })
+            .map(|(stat, time)| stat != self.status && time + SUBSCRIPTION_CHANGE_BACKOFF > Instant::now())
             .unwrap_or(false)
     }
 }
